#define MARCH_PREC 11

uniform float iGlobalTime;
uniform vec2 iResolution;
uniform sampler2D iChannel0;
uniform sampler2D iChannel1;

float rand(vec2 co){
    return fract(sin(dot(co, vec2(12.9898, 78.233))) * 43758.5453);
}

float pallo(vec3 pos, vec3 coord, float r){
    return length(pos - coord) - r;
}

float scene(vec3 pos){
    return min(
        pallo(pos, vec3(0.0, 0.0, (sin(iGlobalTime / 1.5) * 1.5 + 2.0)), 0.8),
        min(
            pallo(pos, vec3((cos(iGlobalTime) * 1.8), (sin(iGlobalTime / 1.1) * 1.8), 1.5), 0.3),
            min(
                pallo(pos, vec3((sin(iGlobalTime / 1.3) * 0.6), (cos(iGlobalTime / 1.6) * 1.4), (sin(iGlobalTime / 1.6) * 1.8 + 1.5)), 0.125),
                pallo(pos, vec3((cos(iGlobalTime / 1.3) * 0.6), (sin(iGlobalTime / 1.6) * 1.4), (cos(iGlobalTime / 1.6) * 1.8 + 1.5)), 0.125)
            )
        )
    );
}

float march(vec3 dir, vec3 cam){
    vec3 curPos = cam;
    for(int i=0; i<MARCH_PREC; i++){
        float dist = scene(curPos);
        curPos += dist * dir;
    }
    return length(curPos - cam);
}

vec3 text() {
    vec2 uv = gl_FragCoord.xy/iResolution.xy;

    float t1 = clamp(-9.0+iGlobalTime, 0.0, 1.0);
    float t2 = clamp(-13.0+iGlobalTime, 0.0, 1.0);

    float a1 = texture2D(iChannel0, uv).r*t1;
    float a2 = texture2D(iChannel1, uv).r*t2;

    return (vec3(a1+a2)+(gl_FragColor.rgb*(1.0-a1+a2)));
}

void main()
{
    float t = iGlobalTime;
    vec2 uv = (gl_FragCoord.xy / iResolution.yy);
    uv.y -= 0.5;
    uv.x -= (iResolution.x/2.0/iResolution.y);
    vec3 ray = normalize(vec3(uv.x, uv.y, 1.0));
    vec3 cam = vec3(0.0, 0.0, -3.0);
    float l = clamp(iGlobalTime/4.0, 0.0, 1.0) / sqrt(march(ray, cam));
    float noise = (rand(uv + iGlobalTime) * 0.034);
    gl_FragColor = vec4((l * l + noise), (l + noise), (l * 0.8 + noise), 0.2);     //Cyan
//    vec3 rgbCol = vec3(l + noise); gl_FragColor = vec4(rgbCol, 0.2);                     //B&W

    gl_FragColor.rgb = text();

    gl_FragColor.rgb *= mod(gl_FragCoord.y, 2.0);

    gl_FragColor.rgb *= clamp(iGlobalTime, 0.0, 1.0);
    gl_FragColor.rgb *= clamp(11.0-iGlobalTime*0.5, 0.0, 1.0);
}
