uniform float iGlobalTime;
uniform vec2 iResolution;
uniform sampler2D iChannel0;

#define MARCH 8

float rand(vec2 co){
    return fract(sin(dot(co, vec2(12.9898, 78.233))) * 43758.5453);
}

float ball(vec3 position, vec3 coord, float r)
{
    return length(position - coord) - r;
}

float scene(vec3 position)
{
    return min(
                  ball(position, vec3(mod(iGlobalTime*0.6+0.2, iResolution.x/iResolution.y*6.0)-iResolution.x/iResolution.y*3.0, 0.25, (1.0 + (sin(iGlobalTime)))), 0.2),
                  min(
                      ball(position, vec3(mod(iGlobalTime*0.9+4.6, iResolution.x/iResolution.y*6.0)-iResolution.x/iResolution.y*3.0, -0.5, (1.0 + (sin(iGlobalTime)))), 0.2),
                      min(
                          ball(position, vec3(mod(iGlobalTime*1.5+0.9, iResolution.x/iResolution.y*6.0)-iResolution.x/iResolution.y*3.0, 0.75, (1.0 + (sin(iGlobalTime)))), 0.2),
                          ball(position, vec3(mod(iGlobalTime*2.0+3.2, iResolution.x/iResolution.y*6.0)-iResolution.x/iResolution.y*3.0, 1.0, (1.0 + (sin(iGlobalTime)))), 0.2)
                      )
                  )
              );
}

float march(vec3 ray, vec3 cam)
{
    vec3 position = cam;
    for(int i=0; i<MARCH; i++)
    {
        float dist = scene(position);
        position += dist * ray;
    }
    return length(position - cam);
}

float raymarcher()
{
    vec2 pos = ((gl_FragCoord.xy/iResolution.yy) - vec2((iResolution.x/iResolution.y) * 0.5, 0.5));
    
    vec3 cam = vec3(0.0, 0.0, -2.0);
    vec3 ray = normalize(vec3(pos, 1.0));
    ray.x += (rand(pos + iGlobalTime*0.0002)*(0.001*iGlobalTime*iGlobalTime))-0.01;
    ray.y += (rand(pos + iGlobalTime*0.0002)*(0.001*iGlobalTime*iGlobalTime))-0.01;
    float l = (1.0 / sqrt(march(ray, cam)));
    return l;
}

void main()
{
    float l = raymarcher();
    vec3 c = vec3(0.8, 0.6, 0.0);
    gl_FragColor = vec4(c*l, 0.1);

    vec4 frame = texture2D(iChannel0, gl_FragCoord.xy/iResolution.xy);
    gl_FragColor.rgb = (frame.rgb*frame.a+(gl_FragColor.rgb*(1.0-frame.a)));

    gl_FragColor.rgb *= clamp(iGlobalTime*0.5, 0.0, 1.0);
    gl_FragColor.rgb *= clamp(18.0-iGlobalTime, 0.0, 1.0);
}
